#ifndef __DIGI_DOC_CFG_H__
#define __DIGI_DOC_CFG_H__
//==================================================
// FILE:	DigiDocCfonfig.h
// PROJECT:     Digi Doc
// DESCRIPTION: Digi Doc functions for configuration management
// AUTHOR:  Veiko Sinivee, S|E|B IT Partner Estonia
//==================================================
// Copyright (C) AS Sertifitseerimiskeskus
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// GNU Lesser General Public Licence is available at
// http://www.gnu.org/copyleft/lesser.html
//==========< HISTORY >=============================
//      08.01.2004      Veiko Sinivee
//                      Creation
//      20.03.2004      Added functions createOrReplacePrivateConfigItem()
//                      writeConfigFile() and writePrivateConfigFile()
//      20.03.2004      changed function notarizeSignature to check for PKCS12 arguments
//==================================================

#include <DigiDocLib.h>

#ifdef  __cplusplus
extern "C" {
#endif

#if defined(EXPORT_LIB_TO_DLL) && defined(WIN32)
  #define EXP_OPTION __declspec(dllexport)
#else
  #define EXP_OPTION
#endif

#include <openssl/x509.h>


// item type
#define ITEM_TYPE_UNKNOWN   0
#define ITEM_TYPE_GLOBAL    1
#define ITEM_TYPE_PRIVATE   2

// used to mark modified items to then store all together in private config file
#define ITEM_STATUS_UNKNOWN 0
#define ITEM_STATUS_OK      1
#define ITEM_STATUS_MODIFIED 2

  //--------------------------------------------------
  // Initializes configuration store
  //--------------------------------------------------
  EXP_OPTION int initConfigStore();

  //--------------------------------------------------
  // Cleans memory of configuration store
  //--------------------------------------------------
  EXP_OPTION void cleanupConfigStore();

  //--------------------------------------------------
  // Adds a new configration item
  // key - items key
  // value - items value
  // type - item type
  // status - item status
  // returns ERR_OK on success
  //--------------------------------------------------
  EXP_OPTION int addConfigItem(const char* key, const char* value, int type, int status);

  //--------------------------------------------------
  // Adds a new private configration item or modifies
  // an existing one
  // key - items key
  // value - items value
  // returns ERR_OK on success
  //--------------------------------------------------
  EXP_OPTION int createOrReplacePrivateConfigItem(const char* key, const char* value);

  //--------------------------------------------------
  // Finds a new configration items value by key
  // key - items key
  // returns value of config item or NULL if not found
  //--------------------------------------------------
  EXP_OPTION const char* ConfigItem_lookup(const char* key);

  //--------------------------------------------------
  // Finds a numeric configration items value by key
  // key - items key
  // defValue - default value to be returned
  // returns value of config item or defValue if not found
  //--------------------------------------------------
  EXP_OPTION int ConfigItem_lookup_int(const char* key, int defValue);

  //--------------------------------------------------
  // Finds a bolean configration items value by key
  // key - items key
  // defValue - default value to be returned
  // returns value of config item or defValue if not found
  //--------------------------------------------------
  EXP_OPTION int ConfigItem_lookup_bool(const char* key, int defValue);

  //--------------------------------------------------
  // Finds a new configration items value by key
  // key - items key
  // returns value of config item or NULL if not found
  //--------------------------------------------------
  //EXP_OPTION const char* ConfigItem_lookup_str(const char* key, const char* defValue);

  //--------------------------------------------------
  // Reads and parses configuration file
  // fileName - configuration file name
  // type - type of config file global/private
  // return error code or 0 for success
  //--------------------------------------------------
  EXP_OPTION int readConfigFile(const char* fileName, int type);

  //--------------------------------------------------
  // Writes a configuration file
  // fileName - configuration file name
  // type - type of config file global/private
  // return error code or 0 for success
  //--------------------------------------------------
  EXP_OPTION int writeConfigFile(const char* fileName, int type);

  //--------------------------------------------------
  // Saves all private config items in correct file
  // return error code or 0 for success
  //--------------------------------------------------
  EXP_OPTION int writePrivateConfigFile();

  //--------------------------------------------------
  // Sets a new name for private config file. Can be
  // used to override default of env(HOME)/.digidoc.conf
  // Use NULL to restore default value
  //--------------------------------------------------
  EXP_OPTION void setPrivateConfigFile(const char* fileName);

  //--------------------------------------------------
  // Finds CA certificate of the given certificate
  // ppCA - address of found CA
  // pCert - certificate whose CA we are looking for
  // return error code or 0 for success
  //--------------------------------------------------
  EXP_OPTION int findCAForCertificate(X509** ppCA, const X509* pCert);

  //--------------------------------------------------
  // Finds CA certificate by CN
  // ppCA - address of found CA
  // szCN - CA certs common name
  // return error code or 0 for success
  //--------------------------------------------------
  EXP_OPTION int findCAForCN(X509** ppCA, const char* szCN);

  //--------------------------------------------------
  // Finds CA chain 
  // ppChain - address of cert pointer array
  // nMaxChain - cert pointer arrary length
  // szCN - CN of the first CA cert (not the child cert!)
  // return error code or 0 for success
  //--------------------------------------------------
  EXP_OPTION int findCAChainForCN(X509** ppChain, int* nMaxChain, const char* szCN);

  //--------------------------------------------------
  // Finds Responders certificate by CN
  // ppResp - address of found cert
  // szCN - Responder certs common name
  // return error code or 0 for success
  //--------------------------------------------------
  EXP_OPTION int findResponderForCN(X509** ppResp, const char* szCN);

  //--------------------------------------------------
  // Finds Responder certificates CA certs CN
  // caCN - buffer for responders CA CN
  // szCN - responder certs common name
  // return error code or 0 for success
  //--------------------------------------------------
  EXP_OPTION int findResponderCAByCN(char* caCN, const char* szCN);

  //------------------------------------------
  // Get a notary confirmation for signature
  // pSigDoc - signed document pointer
  // pSigInfo - signature to notarize
  // returns error code
  //------------------------------------------
  int notarizeSignature(SignedDoc* pSigDoc, SignatureInfo* pSigInfo);

  //--------------------------------------------------
  // Signs the document and gets configrmation
  // pSigDoc - signed document pointer
  // ppSigInfo - address of new signature pointer
  // pin - smart card PIN
  // manifest - manifest / resolution (NULL)
  // city - signers city (NULL)
  // state - signers state (NULL)
  // zip - signers postal code (NULL)
  // country - signers country (NULL)
  //--------------------------------------------------
  EXP_OPTION int signDocument(SignedDoc* pSigDoc, SignatureInfo** ppSigInfo,
		   const char* pin, const char* manifest,
		   const char* city, const char* state,
		   const char* zip, const char* country);

  //--------------------------------------------------
  // Verify this notary
  // pSigDoc - signed document pointer
  // pNotInfo - notary to verify
  // returns error code
  //--------------------------------------------------
  int verifyNotary(SignedDoc* pSigDoc, NotaryInfo* pNotInfo);

  //--------------------------------------------------
  // Verify this signature and it's notary
  // pSigDoc - signed document pointer
  // pSigInfo - signature to verify
  // szFileName - input digidoc filename
  // returns error code
  //--------------------------------------------------
  EXP_OPTION int verifySignatureAndNotary(SignedDoc* pSigDoc, SignatureInfo* pSigInfo, const char* szFileName);

  //--------------------------------------------------
  // Extract common name from cert DN or responder id
  // src - DN
  // dest - buffer for CN
  //--------------------------------------------------
  int findCN(char* src, char* dest);


#ifdef  __cplusplus
}
#endif


#endif // __DIGI_DOC_CFG_H__
